@@ -1,5 +1,5 @@
 from websocket import WebSocketApp, create_connection
-from .support import TickLiveData, TouchlineData
+from .support import TickLiveData, MinLiveData, TouchlineData
 
 from threading import Thread
 
@@ -39,10 +39,10 @@ class LiveClient(WebSocketApp):
         self.remaining_symbols = 0
         self.valid_until = ''
         self.contract_mapping = {}
+        self.subscription_type = ''
         self.parent_app = parent_app
 
     def on_open_func(self):
-        # print(f"Opening WS...")
         pass
 
     def on_msg_func(self, message):
@@ -53,12 +53,18 @@ class LiveClient(WebSocketApp):
             trade = msg['trade']
             self.handle_trade_data(trade)
         elif 'bidask' in msg.keys():
-            pass
+            bidask = msg['bidask']
+            self.handle_bid_ask_data(bidask)
+        elif any(['min' in key for key in msg.keys()]):
+            bar_key = next(key for key in msg.keys() if 'min' in key)
+            bar_data = msg[bar_key]
+            self.handle_bar_data(bar_data)
 
     def handle_message_data(self, msg):
         # print(f"Truedata has this message for you -> {msg['message']}")
         if msg['message'] == 'TrueData Real Time Data Service':  # Connection success message
-            print(f"You have subscribed for {msg['maxsymbols']} symbols across {msg['segments']} until {msg['validity']}...")
+            print(f"You have subscribed for {msg['maxsymbols']} symbols across {msg['segments']} until {msg['validity']} with type of stream as {msg['subscription']}...")
+            self.subscription_type = msg['subscription']
         if msg['message'] == 'invalid request':
             print('raising request error')
             raise TDInvalidRequestError
@@ -87,8 +93,6 @@ class LiveClient(WebSocketApp):
     def handle_trade_data(self, trade_tick):
         try:
             symbol = self.contract_mapping[int(trade_tick[0])]
-            # print(f'{Style.BRIGHT}{Fore.GREEN} This symbol({symbol}) is tied to req_ids of {self.parent_app.symbol_mkt_id_map[symbol]}...{Style.RESET_ALL}')
-            # print(f"{trade_tick} || {type(trade_tick)}")
             for ticker_id in self.parent_app.symbol_mkt_id_map[symbol]:
                 # Assigning new data
                 self.parent_app.live_data[ticker_id].symbol_id = int(trade_tick[0])
@@ -113,26 +117,68 @@ class LiveClient(WebSocketApp):
                 self.parent_app.live_data[ticker_id].change_perc = self.parent_app.live_data[ticker_id].change * 100 / self.parent_app.live_data[ticker_id].prev_day_close
                 try:
                     self.parent_app.live_data[ticker_id].best_bid_price = float(trade_tick[9])
-                    self.parent_app.live_data[ticker_id].best_bid_qty = float(trade_tick[10])
+                    self.parent_app.live_data[ticker_id].best_bid_qty = int(trade_tick[10])
                     self.parent_app.live_data[ticker_id].best_ask_price = float(trade_tick[11])
-                    self.parent_app.live_data[ticker_id].best_ask_qty = float(trade_tick[12])
+                    self.parent_app.live_data[ticker_id].best_ask_qty = int(trade_tick[12])
                 except (IndexError, ValueError, TypeError):
                     del self.parent_app.live_data[ticker_id].best_bid_price
                     del self.parent_app.live_data[ticker_id].best_bid_qty
                     del self.parent_app.live_data[ticker_id].best_ask_price
                     del self.parent_app.live_data[ticker_id].best_ask_qty
+        except KeyError:
+            print(f'{Style.BRIGHT}{Fore.RED}This symbol is not tied to any req_id...{Style.RESET_ALL}')
 
+    def handle_bid_ask_data(self, bidask_tick):
+        try:
+            symbol = self.contract_mapping[int(bidask_tick[0])]
+            for ticker_id in self.parent_app.symbol_mkt_id_map[symbol]:
+                self.parent_app.live_data[ticker_id].symbol_id = int(bidask_tick[0])
+                self.parent_app.live_data[ticker_id].timestamp = datetime.strptime(bidask_tick[1], '%Y-%m-%dT%H:%M:%S')
+                self.parent_app.live_data[ticker_id].best_bid_price = float(bidask_tick[2])
+                self.parent_app.live_data[ticker_id].best_bid_qty = int(bidask_tick[3])
+                self.parent_app.live_data[ticker_id].best_ask_price = float(bidask_tick[4])
+                self.parent_app.live_data[ticker_id].best_ask_qty = int(bidask_tick[5])
+        except KeyError:
+            print(f'{Style.BRIGHT}{Fore.RED}This symbol is not tied to any req_id...{Style.RESET_ALL}')
+
+    def handle_bar_data(self, bar_data):
+        try:
+            symbol = self.contract_mapping[int(bar_data[0])]
+            for ticker_id in self.parent_app.symbol_mkt_id_map[symbol]:
+                # Assigning new data
+                self.parent_app.live_data[ticker_id].symbol_id = int(bar_data[0])
+                self.parent_app.live_data[ticker_id].timestamp = datetime.strptime(bar_data[1], '%Y-%m-%dT%H:%M:%S')
+                self.parent_app.live_data[ticker_id].symbol = symbol
+                self.parent_app.live_data[ticker_id].open = float(bar_data[2])
+                self.parent_app.live_data[ticker_id].high = bar_high = float(bar_data[3])
+                if bar_high > self.parent_app.live_data[ticker_id].day_high:
+                    self.parent_app.live_data[ticker_id].day_high = self.parent_app.touchline_data[ticker_id].high = bar_high
+                self.parent_app.live_data[ticker_id].low = bar_low = float(bar_data[4])
+                if bar_low < self.parent_app.live_data[ticker_id].day_low:
+                    self.parent_app.live_data[ticker_id].day_low = self.parent_app.touchline_data[ticker_id].low = bar_low
+                self.parent_app.live_data[ticker_id].close = self.parent_app.touchline_data[ticker_id].ltp = float(bar_data[5])
+                self.parent_app.live_data[ticker_id].volume = float(bar_data[6])
+                self.parent_app.live_data[ticker_id].oi = float(bar_data[7])
+                # Calculating addn data
+                self.parent_app.live_data[ticker_id].oi_change = self.parent_app.live_data[ticker_id].oi - self.parent_app.live_data[ticker_id].prev_day_oi
+                self.parent_app.live_data[ticker_id].oi_change_perc = self.parent_app.live_data[ticker_id].oi_change * 100 / self.parent_app.live_data[ticker_id].prev_day_oi
+                self.parent_app.live_data[ticker_id].change = self.parent_app.live_data[ticker_id].close - self.parent_app.live_data[ticker_id].prev_day_close
+                self.parent_app.live_data[ticker_id].change_perc = self.parent_app.live_data[ticker_id].change * 100 / self.parent_app.live_data[ticker_id].prev_day_close
         except KeyError:
             print(f'{Style.BRIGHT}{Fore.RED}This symbol is not tied to any req_id...{Style.RESET_ALL}')
 
 
 class HistoricalWebsocket:
-    def __init__(self, login_id, password, url, historical_port):
+    def __init__(self, login_id, password, url, historical_port, broker_token):
         self.login_id = login_id
         self.password = password
         self.url = url
         self.historical_port = historical_port
-        self.hist_socket = create_connection(f"wss://{self.url}:{self.historical_port}?user={self.login_id}&password={self.password}")
+        self.broker_token = broker_token
+        broker_append = ''
+        if self.broker_token is not None:
+            broker_append = f'&brokertoken={self.broker_token}'
+        self.hist_socket = create_connection(f"wss://{self.url}:{self.historical_port}?user={self.login_id}&password={self.password}{broker_append}")
         welcome_msg = self.hist_socket.recv()
         print(f"Connected successfully to '{json.loads(welcome_msg)['message']}'")
 
@@ -176,32 +222,27 @@ class HistoricalWebsocket:
         count = 0
         for j in hist_data:
             try:  # TODO: Remove HOT-FIX because of any erroneous data points
+                try:
+                    oi = int(j[3])
+                except TypeError:  # If OI is None
+                    print(f'{Style.BRIGHT}{Fore.RED} {contract} erred with {j}...{Style.RESET_ALL} \n\t {hist_data[count - 2]} \n\t {hist_data[count - 1]} \n\t {hist_data[count]} \n\t {hist_data[count + 1]} \n\t {hist_data[count + 2]}')
+                    if j[3] is None:
+                        oi = int(hist_data[count - 1][3])
+                        # You can also use prev_datapoint_oi = data_list[-1][3]
                 data_list.append({'time': datetime.strptime(j[0], '%Y-%m-%dT%H:%M:%S'),
                                   'ltp': float(j[1]),
                                   'volume': int(j[2]),
-                                  'oi': int(j[3]),
+                                  'oi': oi,
                                   'bid': float(j[4]),
                                   'bid_qty': int(j[5]),
                                   'ask': float(j[6]),
                                   'ask_qty': int(j[7])})
-            except TypeError:
-                print(f'{Style.BRIGHT}{Fore.RED} {contract} erred with {j}...{Style.RESET_ALL} \n\t {hist_data[count - 2]} \n\t {hist_data[count - 1]} \n\t {hist_data[count]} \n\t {hist_data[count + 1]} \n\t {hist_data[count + 2]}')
-                # If OI is None
-                if j[3] is None:
-                    prev_datapoint_oi = int(hist_data[count-1][3])
-                    data_list.append({'time': datetime.strptime(j[0], '%Y-%m-%dT%H:%M:%S'),
-                                      'ltp': float(j[1]),
-                                      'volume': int(j[2]),
-                                      'oi': prev_datapoint_oi,
-                                      'bid': float(j[4]),
-                                      'bid_qty': int(j[5]),
-                                      'ask': float(j[6]),
-                                      'ask_qty': int(j[7])})
-                    # Even here catch missing bid ask
+            except IndexError:  # No bid-ask data
+                data_list.append({'time': datetime.strptime(j[0], '%Y-%m-%dT%H:%M:%S'),
+                                  'ltp': float(j[1]),
+                                  'volume': int(j[2]),
+                                  'oi': oi})
                 continue
-            except IndexError:
-                # Missing bid-ask data
-                pass
             count = count + 1
         return data_list
 
@@ -235,7 +276,7 @@ class TD:
         t = Thread(target=self.connect_thread, args=())
         t.start()
         if self.connect_historical:
-            self.historical_websocket = HistoricalWebsocket(self.login_id, self.password, self.url, self.historical_port)
+            self.historical_websocket = HistoricalWebsocket(self.login_id, self.password, self.url, self.historical_port, self.broker_token)
 
     def connect_thread(self):
         self.live_websocket.run_forever()
@@ -364,14 +405,22 @@ class TD:
             print(f"{Style.BRIGHT}{Fore.RED}Invalid req_id datatype...{Style.RESET_ALL}")
             raise TDLiveDataError
         for j in range(0, len(req_ids)):
+            resolved_contract = resolved_contracts[j].upper()
             self.touchline_data[req_ids[j]] = TouchlineData()
-            self.live_data[req_ids[j]] = TickLiveData(resolved_contracts[j])
+            if self.live_websocket.subscription_type == 'tick':
+                self.live_data[req_ids[j]] = TickLiveData(resolved_contract)
+            elif 'min' in self.live_websocket.subscription_type:
+                self.live_data[req_ids[j]] = MinLiveData(resolved_contract)
             try:
-                self.symbol_mkt_id_map[resolved_contracts[j].upper()].append(req_ids[j])
+                self.symbol_mkt_id_map[resolved_contract].append(req_ids[j])
             except KeyError:
-                self.symbol_mkt_id_map[resolved_contracts[j].upper()] = [req_ids[j]]
+                self.symbol_mkt_id_map[resolved_contract] = [req_ids[j]]
         self.live_websocket.send(f'{{"method": "addsymbol", "symbols": {json.dumps(resolved_contracts)}}}')
         return req_ids
 
-    def stop_live_data(self, contract):
-        self.live_websocket.send(f'{{"method": "removesymbol", "symbols": ["{contract}"]}}')
+    def stop_live_data(self, contracts):
+        self.live_websocket.send(f'{{"method": "removesymbol", "symbols": {json.dumps(contracts)}}}')
+        for contract in contracts:
+            for req_id in self.symbol_mkt_id_map[contract.upper()]:
+                del self.live_data[req_id]
+            del self.symbol_mkt_id_map[contract.upper()]
