@@ -3,42 +3,49 @@ from colorama import Style, Fore
 
 class TickLiveData:
     def __init__(self, symbol):
+        # --Common Variables
         self.timestamp = None
-        # self.exchange = 'NSE'
         self.symbol = symbol
         self.symbol_id = None
-        self.ltp = None
-        self.best_bid_price = None
-        self.best_bid_qty = None
-        self.best_ask_price = None
-        self.best_ask_qty = None
-        self.volume = None
-        self.atp = None
-        self.oi = None
-        self.ttq = None
-        self.special_tag = ""
         self.day_high = None
         self.day_low = None
         self.day_open = None
         self.prev_day_close = None
+        self.prev_day_oi = None
+        self.oi = None
         self.change = None
         self.change_perc = None
-        self.prev_oi = None
         self.oi_change = None
         self.oi_change_perc = None
+        self.volume = None
+        # --Obj specific
+        self.ltp = None
+        self.atp = None
+        self.ttq = None
+        self.special_tag = ""
         self.tick_seq = None
-        # For level 2 and level 3 data
+        self.best_bid_price = None
+        self.best_bid_qty = None
+        self.best_ask_price = None
+        self.best_ask_qty = None
+        # -- Unused
+        # self.exchange = 'NSE'
+        # -For level 2 and level 3 data
         # self.bids = []
         # self.asks = []
 
     def __eq__(self, other):
         res = True
-        if self.tick_seq != other.tick_seq\
-                or self.best_bid_price != other.best_bid_price\
-                or self.best_bid_qty != other.best_bid_qty\
-                or self.best_ask_price != other.best_ask_price\
-                or self.best_ask_qty != other.best_ask_qty:
-            res = False
+        try:
+            if self.tick_seq != other.tick_seq:
+                res = False
+            elif self.best_bid_price != other.best_bid_price\
+                    or self.best_bid_qty != other.best_bid_qty\
+                    or self.best_ask_price != other.best_ask_price\
+                    or self.best_ask_qty != other.best_ask_qty:
+                res = False
+        except AttributeError:
+            pass
         return res
 
     def __str__(self):
@@ -58,34 +65,35 @@ class TickLiveData:
 
 class MinLiveData:
     def __init__(self, symbol):
+        # --Common Variables
         self.timestamp = None
-        # self.exchange = 'NSE'
         self.symbol = symbol
         self.symbol_id = None
-        self.ltp = None
-        self.best_bid_price = None
-        self.best_bid_qty = None
-        self.best_ask_price = None
-        self.best_ask_qty = None
-        self.volume = None
-        self.atp = None
-        self.oi = None
-        self.total_volume = None
-        self.special_tag = ""
         self.day_high = None
         self.day_low = None
         self.day_open = None
         self.prev_day_close = None
+        self.prev_day_oi = None
+        self.oi = None
         self.change = None
         self.change_perc = None
-        self.prev_day_oi = None
         self.oi_change = None
         self.oi_change_perc = None
-        self.tick_seq = None
-        # For level 2 and level 3 data
+        self.volume = None
+        # --Obj specific
+        self.open = None
+        self.high = None
+        self.low = None
+        self.close = None
+        # --Unused
+        # self.exchange = 'NSE'
+        # -For level 2 and level 3 data
         # self.bids = []
         # self.asks = []
 
+    def __str__(self):
+        return str(self.__dict__)
+
 
 class TouchlineData:
     def __init__(self):
